@@ -78,11 +78,16 @@ namespace apg {
             }
         }
 
-        bool getcell(uint64_t x, uint64_t y) {
+        bool getcell(uint64_t x, uint64_t y) const {
             int32_t u = (uint32_t) (x >> 3);
             int32_t v = (uint32_t) (y >> 3);
-            uint64_t w = (world[std::make_pair(u, v)] >> ((x & 7) | ((y & 7) << 3)));
-            return (w & 1);
+            auto it = world.find({u, v});
+            if (it == world.end()) {
+                return false;
+            } else {
+                uint64_t w = (it->second >> ((x & 7) | ((y & 7) << 3)));
+                return (w & 1);
+            }
         }
 
         void setcell(int64_t x, int64_t y, bool newval) {
@@ -172,10 +177,10 @@ namespace apg {
                 tile &= it->second;
 
                 if (tile == 0) {
-                    world.erase(it++);
+                    it = world.erase(it);
                 } else {
                     it->second = tile;
-                    it++;
+                    ++it;
                 }
             }
         }
@@ -251,22 +256,12 @@ namespace apg {
             }
         }
 
-        uint64_t population() {
-
+        uint64_t population() const {
             uint64_t pop = 0;
-            std::map<std::pair<int32_t, int32_t>, uint64_t>::iterator it;
-            for (it = world.begin(); it != world.end(); /* no increment */ ) {
-                uint64_t tile = it->second;
-                if (tile == 0) {
-                    // Empty tile; erase:
-                    world.erase(it++);
-                } else {
-                    pop += __builtin_popcountll(tile);
-                    it++;
-                }
+            for (const auto& kv : world) {
+                pop += __builtin_popcountll(kv.second);
             }
             return pop;
-
         }
 
         int64_t get_tldiag() {
@@ -277,7 +272,7 @@ namespace apg {
             for (it = world.begin(); it != world.end(); /* no increment */ ) {
                 uint64_t tile = it->second;
                 if (tile == 0) {
-                    world.erase(it++);
+                    it = world.erase(it);
                 } else {
                     int64_t diagonal = ((int64_t) it->first.first ) * 8;
                     diagonal += ((int64_t) it->first.second ) * 8;
@@ -285,14 +280,14 @@ namespace apg {
                         int64_t dz = uint64_tl(tile);
                         if (diagonal + dz < tldiag) { tldiag = diagonal + dz; }
                     }
-                    it++;
+                    ++it;
                 }
             }
 
             return tldiag;
         }
 
-        bool getdbox(int64_t *bbox) {
+        bool getdbox(int64_t *dbox) const {
 
             // These are set outside the admissible range:
             int64_t tl =    20000000000000ll;
@@ -302,58 +297,50 @@ namespace apg {
 
             bool nonempty = false;
 
-            std::map<std::pair<int32_t, int32_t>, uint64_t>::iterator it;
-            for (it = world.begin(); it != world.end(); /* no increment */ ) {
-                uint64_t tile = it->second;
-                if (tile == 0) {
-                    // Empty tile; erase:
-                    world.erase(it++);
-                } else {
-                    // Only determine bounding box if bbox is a valid pointer:
-                    if (bbox != 0) {
-                        int64_t abscissa = ((int64_t) it->first.first ) * 8;
-                        int64_t ordinate = ((int64_t) it->first.second) * 8;
-                        int64_t major = abscissa + ordinate;
-                        int64_t minor = abscissa - ordinate;
+            for (const auto& kv : world) {
+                const auto& coords = kv.first;
+                uint64_t tile = kv.second;
+                if (tile != 0) {
+                    int64_t abscissa = ((int64_t) coords.first ) * 8;
+                    int64_t ordinate = ((int64_t) coords.second) * 8;
+                    int64_t major = abscissa + ordinate;
+                    int64_t minor = abscissa - ordinate;
 
-                        if (major < tl) {
-                            int64_t dx = uint64_tl(tile);
-                            if (major + dx < tl) { tl = major + dx; }
-                        }
+                    if (major < tl) {
+                        int64_t dx = uint64_tl(tile);
+                        if (major + dx < tl) { tl = major + dx; }
+                    }
 
-                        if (major + 14 > br) {
-                            int64_t dx = uint64_br(tile);
-                            if (major + dx > br) { br = major + dx; }
-                        }
+                    if (major + 14 > br) {
+                        int64_t dx = uint64_br(tile);
+                        if (major + dx > br) { br = major + dx; }
+                    }
 
-                        if (minor < bl) {
-                            int64_t dy = uint64_bl(tile);
-                            if (minor + dy < bl) { bl = minor + dy; }
-                        }
+                    if (minor < bl) {
+                        int64_t dy = uint64_bl(tile);
+                        if (minor + dy < bl) { bl = minor + dy; }
+                    }
 
-                        if (minor + 14 > tr) {
-                            int64_t dy = uint64_tr(tile);
-                            if (minor + dy > tr) { tr = minor + dy; }
-                        }
+                    if (minor + 14 > tr) {
+                        int64_t dy = uint64_tr(tile);
+                        if (minor + dy > tr) { tr = minor + dy; }
                     }
 
-                    // We've encountered a non-empty tile; proceed:
                     nonempty = true;
-                    it++;
                 }
             }
 
-            if (nonempty && (bbox != 0)) {
-                bbox[0] = tl;
-                bbox[1] = bl - 7;
-                bbox[2] = (1 + br - tl);
-                bbox[3] = (1 + tr - bl);
+            if (nonempty) {
+                dbox[0] = tl;
+                dbox[1] = bl - 7;
+                dbox[2] = (1 + br - tl);
+                dbox[3] = (1 + tr - bl);
             }
 
             return nonempty;
         }
 
-        bool getbbox(int64_t *bbox) {
+        bool getbbox(int64_t *bbox) const {
 
             // These are set outside the admissible range:
             int64_t left =    20000000000000ll;
@@ -363,46 +350,38 @@ namespace apg {
 
             bool nonempty = false;
 
-            std::map<std::pair<int32_t, int32_t>, uint64_t>::iterator it;
-            for (it = world.begin(); it != world.end(); /* no increment */ ) {
-                uint64_t tile = it->second;
-                if (tile == 0) {
-                    // Empty tile; erase:
-                    world.erase(it++);
-                } else {
-                    // Only determine bounding box if bbox is a valid pointer:
-                    if (bbox != 0) {
-                        int64_t abscissa = ((int64_t) it->first.first ) * 8;
-                        int64_t ordinate = ((int64_t) it->first.second) * 8;
+            for (const auto& kv : world) {
+                const auto& coords = kv.first;
+                uint64_t tile = kv.second;
+                if (tile != 0) {
+                    int64_t abscissa = ((int64_t) coords.first ) * 8;
+                    int64_t ordinate = ((int64_t) coords.second) * 8;
 
-                        if (abscissa < left) {
-                            int64_t dx = uint64_left(tile);
-                            if (abscissa + dx < left) { left = abscissa + dx; }
-                        }
+                    if (abscissa < left) {
+                        int64_t dx = uint64_left(tile);
+                        if (abscissa + dx < left) { left = abscissa + dx; }
+                    }
 
-                        if (abscissa + 7 > right) {
-                            int64_t dx = uint64_right(tile);
-                            if (abscissa + dx > right) { right = abscissa + dx; }
-                        }
+                    if (abscissa + 7 > right) {
+                        int64_t dx = uint64_right(tile);
+                        if (abscissa + dx > right) { right = abscissa + dx; }
+                    }
 
-                        if (ordinate < top) {
-                            int64_t dy = uint64_top(tile);
-                            if (ordinate + dy < top) { top = ordinate + dy; }
-                        }
+                    if (ordinate < top) {
+                        int64_t dy = uint64_top(tile);
+                        if (ordinate + dy < top) { top = ordinate + dy; }
+                    }
 
-                        if (ordinate + 7 > bottom) {
-                            int64_t dy = uint64_bottom(tile);
-                            if (ordinate + dy > bottom) { bottom = ordinate + dy; }
-                        }
+                    if (ordinate + 7 > bottom) {
+                        int64_t dy = uint64_bottom(tile);
+                        if (ordinate + dy > bottom) { bottom = ordinate + dy; }
                     }
 
-                    // We've encountered a non-empty tile; proceed:
                     nonempty = true;
-                    it++;
                 }
             }
 
-            if (nonempty && (bbox != 0)) {
+            if (nonempty) {
                 bbox[0] = left;
                 bbox[1] = top;
                 bbox[2] = (1 + right - left);
@@ -412,23 +391,21 @@ namespace apg {
             return nonempty;
         }
 
-        bool getboct(int64_t *boct) {
+        bool getboct(int64_t *boct) const {
             getbbox(boct);
             return getdbox(boct + 4);
         }
 
-        bitworld& clean() {
-            std::map<std::pair<int32_t, int32_t>, uint64_t>::iterator it;
-            for (it = world.begin(); it != world.end(); /* no increment */ ) {
+        void clean() {
+            for (auto it = world.begin(); it != world.end(); /* no increment */ ) {
                 uint64_t tile = it->second;
                 if (tile == 0) {
                     // Empty tile; erase:
-                    world.erase(it++);
+                    it = world.erase(it);
                 } else {
-                    it++;
+                    ++it;
                 }
             }
-            return *this;
         }
 
         bitworld inflate() {
@@ -449,11 +426,10 @@ namespace apg {
         bitworld get1cell() {
 
             bitworld x;
-            std::map<std::pair<int32_t, int32_t>, uint64_t>::iterator it;
-            for (it = world.begin(); it != world.end(); /* no increment */ ) {
+            for (auto it = world.begin(); it != world.end(); /* no increment */ ) {
                 uint64_t tile = it->second;
                 if (tile == 0) {
-                    world.erase(it++);
+                    it = world.erase(it);
                 } else {
                     x.world[it->first] = tile & (~(tile - 1));
                     break;
@@ -467,17 +443,16 @@ namespace apg {
 
             std::pair<int32_t, int32_t> record(-1000000000, -1000000000);
             uint64_t recordcell = 0;
-            std::map<std::pair<int32_t, int32_t>, uint64_t>::iterator it;
-            for (it = world.begin(); it != world.end(); /* no increment */ ) {
+            for (auto it = world.begin(); it != world.end(); /* no increment */ ) {
                 uint64_t tile = it->second;
                 if (tile == 0) {
-                    world.erase(it++);
+                    it = world.erase(it);
                 } else {
                     if (it->first.first + it->first.second > record.first + record.second) {
                         record = it->first;
                         recordcell = tile & (~(tile - 1));
                     }
-                    it++;
+                    ++it;
                 }
             }
             bitworld x;
@@ -548,7 +523,79 @@ namespace apg {
         return planes;
     }
 
-    std::vector<bitworld> rle2vec(std::string rle, RuleMapper *rm) {
+    std::string vec2rle(const std::vector<bitworld>& planes, const RuleMapper& rm) {
+        int N = planes.size();
+        bitworld allPlanes;
+        for (int i=0; i < N; ++i) {
+            allPlanes += planes[i];
+        }
+        int64_t boundingBox[4];
+        if (!allPlanes.getbbox(boundingBox)) {
+            return "!";
+        }
+
+        const int64_t xmin = boundingBox[0];
+        const int64_t xmax = boundingBox[0] + boundingBox[2];
+        const int64_t ymin = boundingBox[1];
+        const int64_t ymax = boundingBox[1] + boundingBox[3];
+
+        int currentColor = 0;
+        int currentRunLength = 0;
+        int sizeAtLastNewline = 0;
+        std::string rle;
+
+        auto encodeColor = [](std::string& rle, int color) {
+            if (color == 0) {
+                rle += '.';
+            } else if (color <= 24) {
+                rle += ('A' + (color-1) % 24);
+            } else {
+                auto it = rle.end();
+                while (color != 0) {
+                    it = rle.insert(it, 'A' + (color-1) % 24);
+                    color /= 24;
+                }
+            }
+        };
+
+        for (int64_t y = ymin; y < ymax; ++y) {
+            for (int64_t x = xmin; x < xmax; ++x) {
+                int colorOfThisCell = 0;
+                for (int i=0; i < N; ++i) {
+                    colorOfThisCell |= planes[i].getcell(x, y) << i;
+                }
+                colorOfThisCell = rm.ll_to_golly(colorOfThisCell);
+                if (rle.size() - sizeAtLastNewline >= 75) {
+                    rle += '\n';
+                    sizeAtLastNewline = rle.size();
+                }
+                if (colorOfThisCell == currentColor) {
+                    ++currentRunLength;
+                } else {
+                    if (currentRunLength != 0) {
+                        if (currentRunLength != 1) {
+                            rle += std::to_string(currentRunLength);
+                        }
+                        encodeColor(rle, currentColor);
+                    }
+                    currentColor = colorOfThisCell;
+                    currentRunLength = 1;
+                }
+            }
+            if (currentRunLength != 0) {
+                if (currentRunLength != 1) {
+                    rle += std::to_string(currentRunLength);
+                }
+                encodeColor(rle, currentColor);
+            }
+            rle += '$';
+            currentRunLength = 0;
+        }
+        rle += '!';
+        return rle;
+    }
+
+    std::vector<bitworld> rle2vec(std::string rle, const RuleMapper& rm) {
 
         if (rle.find('!') == std::string::npos) {
             auto p = rle.find('_');
@@ -579,7 +626,7 @@ namespace apg {
                     colour = colour * 24 + (c - 'A') + 1;
                 }
                 uint64_t bp = 0;
-                if (rm != 0) { colour = rm->golly_to_ll(colour); }
+                colour = rm.golly_to_ll(colour);
                 while (colour > 0) {
                     if (colour & 1) {
                         if (bp >= planes.size()) { planes.resize(bp+1); }
@@ -598,56 +645,58 @@ namespace apg {
         return planes;
     }
 
-    std::vector<bitworld> rle2vec(std::string rle) { return rle2vec(rle, 0); }
+    std::vector<bitworld> rle2vec(const std::string& rle) {
+        return rle2vec(rle, IdentityMapper());
+    }
 
     bitworld _shift_left(const bitworld &a, uint32_t k) {
         bitworld b;
-        std::map<std::pair<int32_t, int32_t>, uint64_t>::const_iterator it;
         uint64_t bitmask = (0x0101010101010101ull << k) - 0x0101010101010101ull;
-        for (it = a.world.begin(); it != a.world.end(); ++it) {
-            b.world[it->first] |= (it->second & (~bitmask)) >> k;
-            int32_t newx = it->first.first - 1;
-            int32_t newy = it->first.second;
-            b.world[std::make_pair(newx, newy)] |= (it->second & bitmask) << (8-k);
+        for (const auto& kv : a.world) {
+            b.world[kv.first] |= (kv.second & (~bitmask)) >> k;
+            int32_t newx = kv.first.first - 1;
+            int32_t newy = kv.first.second;
+            b.world[std::make_pair(newx, newy)] |= (kv.second & bitmask) << (8-k);
         }
-        return b.clean();
+        b.clean();
+        return b;
     }
 
     bitworld _shift_right(const bitworld &a, uint32_t k) {
         bitworld b;
-        std::map<std::pair<int32_t, int32_t>, uint64_t>::const_iterator it;
         uint64_t bitmask = (0x0101010101010101ull << (8-k)) - 0x0101010101010101ull;
-        for (it = a.world.begin(); it != a.world.end(); ++it) {
-            b.world[it->first] |= (it->second & bitmask) << k;
-            int32_t newx = it->first.first + 1;
-            int32_t newy = it->first.second;
-            b.world[std::make_pair(newx, newy)] |= (it->second & (~bitmask)) >> (8-k);
+        for (const auto& kv : a.world) {
+            b.world[kv.first] |= (kv.second & bitmask) << k;
+            int32_t newx = kv.first.first + 1;
+            int32_t newy = kv.first.second;
+            b.world[std::make_pair(newx, newy)] |= (kv.second & (~bitmask)) >> (8-k);
         }
-        return b.clean();
+        b.clean();
+        return b;
     }
 
     bitworld _shift_down(const bitworld &a, uint32_t k) {
         bitworld b;
-        std::map<std::pair<int32_t, int32_t>, uint64_t>::const_iterator it;
-        for (it = a.world.begin(); it != a.world.end(); ++it) {
-            b.world[it->first] |= (it->second << (8*k));
-            int32_t newx = it->first.first;
-            int32_t newy = it->first.second + 1;
-            b.world[std::make_pair(newx, newy)] |= (it->second >> (8*(8-k)));
+        for (const auto& kv : a.world) {
+            b.world[kv.first] |= (kv.second << (8*k));
+            int32_t newx = kv.first.first;
+            int32_t newy = kv.first.second + 1;
+            b.world[std::make_pair(newx, newy)] |= (kv.second >> (8*(8-k)));
         }
-        return b.clean();
+        b.clean();
+        return b;
     }
 
     bitworld _shift_up(const bitworld &a, uint32_t k) {
         bitworld b;
-        std::map<std::pair<int32_t, int32_t>, uint64_t>::const_iterator it;
-        for (it = a.world.begin(); it != a.world.end(); ++it) {
-            b.world[it->first] |= (it->second >> (8*k));
-            int32_t newx = it->first.first;
-            int32_t newy = it->first.second - 1;
-            b.world[std::make_pair(newx, newy)] |= (it->second << (8*(8-k)));
+        for (const auto& kv : a.world) {
+            b.world[kv.first] |= (kv.second >> (8*k));
+            int32_t newx = kv.first.first;
+            int32_t newy = kv.first.second - 1;
+            b.world[std::make_pair(newx, newy)] |= (kv.second << (8*(8-k)));
         }
-        return b.clean();
+        b.clean();
+        return b;
     }
 
     bitworld bleed(const bitworld &a, std::string s) {
@@ -689,7 +738,7 @@ namespace apg {
         return b;
     }
 
-    bitworld fix_topleft(bitworld &a) {
+    bitworld fix_topleft(const bitworld &a) {
         int64_t bbox[4] = {0};
         bool nonempty = a.getbbox(bbox);
         if (nonempty == false) { return a; }
