@@ -1,8 +1,6 @@
-
-
+    #ifdef __AVX512F__
     template<>
         void VTile<44>::copyBoundary3(VTile<44> *n) {
-            #ifdef __AVX512F__
             asm (
                 "vmovdqu64 8(%0), %%zmm6 \n\t"
                 "vmovdqu64 8(%1), %%zmm8 \n\t"
@@ -31,16 +29,10 @@
                 : /* no output operands */ 
                 : "r" (d), "r" (n->d), "r" (apg::__16xfffffffc)
                 : "xmm6", "xmm8", "xmm13", "xmm14", "memory");
-            #else
-            for (int i = 2; i < 46; i++) {
-                d[i] = ((n->d[i] & 0x3ffffffcu) >> 28) | (d[i] & 0xfffffffcu);
-            }
-            #endif
         }
 
     template<>
         void VTile<44>::copyBoundary0(VTile<44> *n) {
-            #ifdef __AVX512F__
             asm (
                 "vmovdqu64 8(%0), %%zmm6 \n\t"
                 "vmovdqu64 8(%1), %%zmm8 \n\t"
@@ -69,13 +61,10 @@
                 : /* no output operands */ 
                 : "r" (d), "r" (n->d), "r" (apg::__16x3fffffff)
                 : "xmm6", "xmm8", "xmm13", "xmm14", "memory");
-            #else
-            for (int i = 2; i < 46; i++) {
-                d[i] = ((n->d[i] & 0x3ffffffcu) << 28) | (d[i] & 0x3fffffffu);
-            }
-            #endif
         }
 
+    #endif
+
     template<>
         inline void VTile<44>::updateTile(upattern<VTile<44>, 28, 44>* owner, int rule, int family, int mantissa) {
 
