@@ -33,8 +33,42 @@ def create_rule(rulestring):
 
     rule_property(rulestring, 'create_rule')
 
+def decode_ltl(rulestring):
+
+    rparts = rulestring.lower().split(',')
+
+    d = {'c': 2, 'm': 1, 'n': 'm'}
+    p = ['r', 'bmin', 'bmax', 'smin', 'smax']
+
+    for r in rparts:
+        if (r[0] >= 'a') and (r[0] <= 'z'):
+            if '..' in r[1:]:
+                a, b = r[1:].split('..')
+                d[r[0] + 'min'] = a
+                d[r[0] + 'max'] = b
+            else:
+                d[r[0]] = r[1:]
+        else:
+            d[[x for x in p if x not in d][0]] = r
+
+    if d['n'] != 'm':
+        raise ValueError("Unsupported LtL neighbourhood type")
+
+    d = {k : int(d[k]) for k in ['r', 'c', 'm', 'smin', 'smax', 'bmin', 'bmax']}
+
+    d['smin'] += (1 - d['m'])
+    d['smax'] += (1 - d['m'])
+
+    rulestring = 'r%db%dt%ds%dt%d' % (d['r'], d['bmin'], d['bmax'], d['smin'], d['smax'])
+    if d['c'] > 2:
+        rulestring = 'g%d%s' % (d['c'], rulestring)
+    return rulestring
+
 def sanirule(rulestring, drop_history=False):
 
+    if ',' in rulestring:
+        rulestring = decode_ltl(rulestring)
+
     if '.' in rulestring:
         try:
             rulestring = rule2files(rulestring)
