@@ -280,6 +280,14 @@ extern "C"
         ppat->getrect(bbox);
     }
 
+    uint64_t GetPatternDigest(void* pat) {
+        auto ppat = reinterpret_cast<apg::pattern*>(pat);
+        if (ppat->empty()) { return 0; }
+        int64_t bbox[4];
+        ppat->getrect(bbox);
+        return ppat->shift(0 - bbox[0], 0 - bbox[1]).digest();
+    }
+
     void GetRuleOfPattern(void* pat, char* buffer) {
         auto ppat = reinterpret_cast<apg::pattern*>(pat);
         std::string s = ppat->getrule();
