@@ -207,7 +207,7 @@ namespace apg {
 
             } else {
 
-                RuleMapper* grm = getMapper(rule);
+                auto grm = getMapper(rule);
                 int linelength = 0;
 
                 std::map<std::pair<uint64_t, uint64_t>, uint64_t> cells;
@@ -258,8 +258,6 @@ namespace apg {
 
                 printrun(outstream, runlength, state2string(lastc), linelength);
                 outstream << "!" << std::endl;
-
-                delete grm;
             }
         }
 
@@ -454,7 +452,7 @@ namespace apg {
 
             std::ostringstream rlestream;
 
-            RuleMapper *grm = new IdentityMapper();
+            auto grm = getMapper("identity");
 
             int golly_numbering = 2;
 
@@ -471,14 +469,14 @@ namespace apg {
                     auto posrule = line.find("rule");
                     if (posrule != std::string::npos) {
                         rule = sanirule(line.substr(posrule + 4));
-                        delete grm; grm = getMapper(rule);
+                        grm = getMapper(rule);
                         // std::cerr << line << " interpreted as " << rule << std::endl;
                     }
                 } else if (line[0] == '#') {
                     if (line.length() >= 2 && (line[1] == 'R' || line[1] == 'G')) {
                         if (line[1] == 'R') {
                             rule = sanirule(line.substr(3));
-                            delete grm; grm = getMapper(rule);
+                            grm = getMapper(rule);
                             // std::cerr << line << " interpreted as " << rule << std::endl;
                         }
                     } else if ((line.length() > 4) && (line.substr(0, 4) == "#CLL")) {
@@ -594,12 +592,15 @@ namespace apg {
             hypernode<I> result_hn;
 
             if (reading_rle) {
-                result_hn = this->fromrle(rlestream.str(), ((golly_numbering & 2) ? grm : 0));
+                if (golly_numbering & 2) {
+                    result_hn = this->fromrle(rlestream.str(), *grm);
+                } else {
+                    result_hn = this->fromrle(rlestream.str(), IdentityMapper());
+                }
             } else {
                 result_hn = hypernode<I>(lastnode, log2size - 4);
             }
 
-            delete grm;
             return result_hn;
         }
 
@@ -1393,12 +1394,12 @@ namespace apg {
                 c ^= ((d + a) << 23) | ((d + a) >> 41);
                 b ^= ((c + d) << 59) | ((c + d) >> 5);
 
-                // Extra non-linearity:
-                b %= 157079632679489783;
-
-                // Now compute a hash:
-                uint64_t h = (65537*a + 257*b + 17*c + 5*d);
+                uint64_t h = a;
+                h = h * 6364136223846793005ull + b;
+                h = h * 6364136223846793005ull + c;
+                h = h * 6364136223846793005ull + d;
                 h ^= (h >> 32);
+
                 memmap->emplace(std::make_pair(hnode.index, hnode.depth), h);
                 return h;
             }
