@@ -290,7 +290,7 @@ namespace apg {
             return fromplanes(cells2vec(n, coords, states));
         }
 
-        hypernode<I> fromrle(std::string s, RuleMapper *rm) {
+        hypernode<I> fromrle(const std::string& s, const RuleMapper& rm) {
             if (s != "") {
                 return fromplanes(rle2vec(s, rm));
             } else {
@@ -298,7 +298,9 @@ namespace apg {
             }
         }
 
-        hypernode<I> fromrle(std::string s) { return fromrle(s, 0); }
+        hypernode<I> fromrle(const std::string& s) {
+            return fromrle(s, IdentityMapper());
+        }
 
         virtual int rulestring_to_integer(std::string rulestring) = 0;
         virtual hypernode<I> iterate_recurse(hypernode<I> hnode, uint64_t mantissa, uint64_t exponent, int rule, int history) = 0;
