@@ -50,6 +50,18 @@ class Pattern(object):
         return ("<Pattern(logdiam=%d, beszel_index=%d, ulqoma_index=%d, rule=%s) owned by %r>" % (logdiam,
             beszel_index, ulqoma_index, rule, self.owner))
 
+    def digest(self):
+
+        return self.lifelib('GetPatternDigest', self.ptr)
+
+    def octodigest(self):
+
+        od = [self.digest()]
+        for o in ["rot270", "rot180", "rot90", "flip_x", "flip_y", "swap_xy", "swap_xy_flip"]:
+            od.append(self(o).digest())
+        od = sorted(set(od))
+        return sum([(2*i+1)*j for (i, j) in enumerate(od)]) & (2**64 - 1)
+
     def advance(self, numgens, exponent=0):
 
         self.lifelib.ensure_limit(exponent, numgens, 'Number of generations')
