@@ -6,12 +6,14 @@ from collections import Sequence, defaultdict
 def get_symmetries(rulestring):
 
     validsyms = ["1x256", "2x128", "4x64", "8x32", "C1"] # asymmetric soups
+    for sz in ["32x32", "64x64", "128x128", "256x256", "512x512", "1k", "2k", "4k", "8k", "Glider8_4_5", "Glider6_5_6"]:
+        validsyms += ["Mateon1_" + sz + "_Test"]
 
     if rulestring[-1].lower() == 'h':
         validsyms += ["C2_4", "C2_1", "C3_1", "C6", "D2_x", "D2_xo", "D4_x4", "D4_x1", "D6_1", "D6_1o", "D12"]
     else:
         validsyms += ["C2_4", "C2_2", "C2_1", "C4_4", "C4_1",
-                        "D2_+2", "D2_+1", "D2_x", "D4_+4", "D4_+2", "D4_+1", "D4_x4", "D4_x1", "D8_4", "D8_1"]
+                        "D2_+2", "D2_+1", "D2_x", "D4_+4", "D4_+2", "D4_+1", "D4_x4", "D4_x1", "D8_4", "D8_1", "D2_+1_gO1s0", "D2_+1_gO1s1", "D2_+1_gO1s2"]
 
     if (re.match('^b0?1?2?3?4?5?6?7?8?s0?1?2?3?4?5?6?7?8?$', rulestring)):
         validsyms += ["G1", "H2_+1", "H2_+2", "H4_+1", "H4_+2", "H4_+4"] # GPU symmetries
