@@ -7,6 +7,8 @@ import atexit
 
 from ..genera import sanirule
 
+from .identify import *
+
 floats = {}
 
 
@@ -25,6 +27,20 @@ class Lifetree(object):
 
         session.lifetree_ptrs.add(self.ptr)
 
+    def load_timeline(self, filename, n):
+
+        import numpy as np
+        frames = np.zeros((n,), dtype=np.uint64, order='C')
+        frames = self.lifelib('LoadTimelineMC', self.ptr, filename, n, frames)
+
+        patterns = []
+
+        for f in frames:
+            if f != 0:
+                patterns.append(Pattern(self.session, (int(f),), self))
+
+        return patterns
+
     def save_timeline(self, patterns, filename, header="", footer="", exponent=0, startgen=0):
 
         import numpy as np
@@ -53,7 +69,36 @@ class Lifetree(object):
         n = self.n_layers
         return tuple([((pattern << (n - 1 - i)) >> (n - 1)) for i in range(n)])
 
+    def loadu(self, filename):
+        '''
+        Load an uncompressed pattern file.
+        '''
+
+        filename = os.path.abspath(filename)
+        metadata = identify_metadata(filename)
+        timeline = False
+
+        if 'frames' in metadata:
+            try:
+                n = int(metadata['frames'][0])
+                timeline = True
+            except:
+                pass
+
+        if timeline:
+            # timeline file
+            return self.load_timeline(filename, n)
+        else:
+            # pattern file
+            pptr = self.lifelib('CreatePatternFromFile', self.ptr, filename)
+            pat = Pattern(self.session, pptr, self)
+            pat.comments = metadata['comments']
+            return pat
+
     def load(self, filename, compressed='deduce', tempfile='tempfile'):
+        '''
+        Load a (possibly compressed) pattern file.
+        '''
 
         filename = os.path.abspath(filename)
         tempfile = os.path.abspath(tempfile)
@@ -62,23 +107,26 @@ class Lifetree(object):
             raise OSError("%s does not exist or is not a regular file" % filename)
 
         if compressed == 'deduce':
-            compressed = (filename[-3:] == '.gz')
+            compressed = identify_compression(filename)
 
         if compressed:
-            import gzip
+
+            from importlib import import_module
+            decompressor = import_module(compressed)
+
             import shutil
-            with gzip.open(filename, 'rb') as f_in:
+            with decompressor.open(filename, 'rb') as f_in:
                 with open(tempfile, 'wb') as f_out:
                     shutil.copyfileobj(f_in, f_out)
-            pptr = self.lifelib('CreatePatternFromFile', self.ptr, tempfile)
+            rt = self.loadu(tempfile)
             try:
                 os.remove(tempfile)
             except OSError:
                 pass
         else:
-            pptr = self.lifelib('CreatePatternFromFile', self.ptr, filename)
+            rt = self.loadu(filename)
 
-        return Pattern(self.session, pptr, self)
+        return rt
 
     def hashsoup(self, rule, symmetry, seed):
 
