@@ -166,7 +166,7 @@ namespace apg {
             unsigned digest_idx = 0;
             unsigned digest_bits = 0;
 
-            auto sha_rand = [&](unsigned requested_bits) {
+            auto sha_rand = [&](unsigned requested_bits) -> uint64_t {
                 uint64_t val = 0;
                 uint64_t ishift = 0;
 
@@ -259,7 +259,7 @@ repeat:
             if (verbose) {
                 for (int y = miny; y <= maxy; y++) {
                     for (int x = minx; x <= maxx; x++) {
-                        std::cout << (bw.getcell(x, y) ? '@' : '.');
+                        std::cout << (bw.getcell(x, y) ? 'o' : '.');
                     }
                     std::cout << std::endl;
                 }
