@@ -1,16 +1,17 @@
 #pragma once
 
-#include "avxlife/uli.h"
 #include <unordered_map>
 #include <string>
 #include <vector>
 #include <iostream>
-#include "bitworld.h"
-#include "avxlife/lifeconsts.h"
+#include <type_traits>
 
+#include "avxlife/uli.h"
+#include "avxlife/lifeconsts.h"
 #include "hashtrees/kivtable.h"
 
-#include <type_traits>
+#include "bitworld.h"
+#include "incubator.h"
 
 namespace apg {
 
@@ -50,9 +51,18 @@ namespace apg {
         indirected_map<uint64_t, T> tiles;
         std::vector<T*> modified;
         std::vector<T*> temp_modified;
+        std::vector<T*> popchanged;
 
         uint64_t tilesProcessed;
         uint64_t gensElapsed;
+        int population;
+        int extremal_mask;
+        int glider_count;
+
+        int64_t max_xpy;
+        int64_t min_xpy;
+        int64_t max_xmy;
+        int64_t min_xmy;
 
         T* coords2ptr(int64_t x, int64_t w) {
             // Returns a pointer to tile x + omega*w.
@@ -77,6 +87,10 @@ namespace apg {
             torus_width = 0;
             torus_height = 0;
             lastmant = 1;
+            population = 0;
+            extremal_mask = 0;
+            glider_count = 0;
+            max_xpy = 0; min_xpy = 0; max_xmy = 0; min_xmy = 0;
         }
 
         upattern(int width, int height) {
@@ -86,6 +100,10 @@ namespace apg {
             torus_width = width / W;
             torus_height = height / H;
             lastmant = 1;
+            population = 0;
+            extremal_mask = 0;
+            glider_count = 0;
+            max_xpy = 0; min_xpy = 0; max_xmy = 0; min_xmy = 0;
 
             if ((width == 0) || (height == 0)) { return; }
 
@@ -103,12 +121,44 @@ namespace apg {
             } 
         }
 
+        int is_extremal(uint64_t x) {
+            #ifdef DISABLE_GLIDERS
+            (void) x;
+            return 0;
+            #else
+            int ext = 0;
+            int64_t tx = (x & 0xffffffffu) - 0x80000000u;
+            int64_t tw = (x >> 32) - 0x80000000u;
+            int64_t xpy = tx * W - (tw * (H + W/2));
+            int64_t xmy = tx * W + (tw * (H - W/2));
+            ext |= ((xpy >= max_xpy) ? 1 : 0);
+            ext |= ((xpy <= min_xpy) ? 2 : 0);
+            ext |= ((xmy >= max_xmy) ? 4 : 0);
+            ext |= ((xmy <= min_xmy) ? 8 : 0);
+            return (ext & extremal_mask);
+            #endif
+        }
+
         T* getNeighbour(T* sqt, int i) {
             T* y = sqt->neighbours[i];
             if (!y) {
                 uint64_t x = sqt->coords + udirections[i];
-                y = &tiles[x];
-                y->coords = x;
+                T** pointer_to_pointer = &(tiles.hashtable[x]);
+                if (*pointer_to_pointer == 0) {
+                    *pointer_to_pointer = tiles.elements.newnode();
+                    (*pointer_to_pointer)->coords = x;
+                    #ifndef DISABLE_GLIDERS
+                    int64_t tx = (x & 0xffffffffu) - 0x80000000u;
+                    int64_t tw = (x >> 32) - 0x80000000u;
+                    int64_t xpy = tx * W - (tw * (H + W/2));
+                    int64_t xmy = tx * W + (tw * (H - W/2));
+                    max_xpy = (xpy > max_xpy) ? xpy : max_xpy;
+                    min_xpy = (xpy < min_xpy) ? xpy : min_xpy;
+                    max_xmy = (xmy > max_xmy) ? xmy : max_xmy;
+                    min_xmy = (xmy < min_xmy) ? xmy : min_xmy;
+                    #endif
+                }
+                y = *pointer_to_pointer;
                 y->neighbours[(i + 3) % 6] = sqt;
                 sqt->neighbours[i] = y;
             }
@@ -198,14 +248,12 @@ namespace apg {
 
         int totalPopulation() {
 
-            int population = 0;
-
-            uint32_t totalnodes = tiles.elements.totalnodes;
-            for (uint32_t it = 0; it != totalnodes; ++it) {
-                T* sqt = tiles.elements.ind2ptr(it);
-                population += sqt->countPopulation();
+            for (auto it = popchanged.begin(); it != popchanged.end(); ++it) {
+                (*it)->countPopulation(this);
             }
 
+            popchanged.clear();
+
             return population;
         }
 
@@ -290,5 +338,55 @@ namespace apg {
     #endif
     #include "classic/utile.h"
 
+    template<int H>
+    void copycells(upattern<VTile<H>, 28, H>* curralgo, incubator<56, H*2>* destalgo) {
+
+        uint32_t totalnodes = curralgo->tiles.elements.totalnodes;
+        for (uint32_t it = 0; it != totalnodes; ++it) {
+            VTile<H> *sqt = curralgo->tiles.elements.ind2ptr(it);
+
+            if (sqt->countPopulation(curralgo) == 0) { continue; }
+
+            int64_t tx = (sqt->coords & 0xffffffffu) - 0x80000000u;
+            int64_t tw = (sqt->coords >> 32) - 0x80000000u;
+
+            bool split = (((2 * tx - tw) & 3) == 3);
+
+            for (int half = 0; half < 1 + split; half++) {
+
+                int mx = 2 * tx - tw + half;
+                int my = -tw;
+
+                int lx = mx % 4;
+                int ly = my % 2;
+
+                if (lx < 0) {lx += 4;}
+                if (ly < 0) {ly += 2;}
+
+                int tx = (mx - lx) / 4;
+                int ty = (my - ly) / 2;
+
+                Incube<56, H*2>* sqt2 = &(destalgo->tiles[std::pair<int, int>(tx, ty)]);
+                sqt2->tx = tx; sqt2->ty = ty;
+
+                for (int i = 0; i < H; i++) {
+                    uint64_t insert;
+                    if (half == 1) {
+                        insert = (sqt->d[i+2] >> 16) & 0x3fff;
+                    } else {
+                        insert = (sqt->d[i+2] >> 2) & (split ? 0x3fff : 0xfffffff);
+                    }
+                    sqt2->d[i + H * ly] |= (insert << (14 * lx));
+                    if (half == 1) {
+                        insert = (sqt->hist[i+2] >> 16) & 0x3fff;
+                    } else {
+                        insert = (sqt->hist[i+2] >> 2) & (split ? 0x3fff : 0xfffffff);
+                    }
+                    sqt2->hist[i + H * ly] |= (insert << (14 * lx));
+                }
+            }
+        }
+    }
+
 } // namespace apg
 
